##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################
"""
Core session tracking SessionData class.

$Id: SessionData.py,v 1.6 2001/04/01 01:25:09 chrism Exp $
"""

__version__='$Revision: 1.6 $'[11:-2]

import time, sys
import Globals
from Persistence import Persistent
import Acquisition
from zLOG import LOG
import ExtensionClass
try:
    from SessioningInterfaces import SessionDataInterface
except:
    # pre-2.3 Zopes don't have an interfaces module
    class SessionDataInterface:
        pass
_marker = []

class SessionData(Persistent, Acquisition.Explicit):
    """ simple session data object """

    __allow_access_to_unprotected_subobjects__ = 1

    __implements__ = SessionDataInterface

    def __init__(self, id, onstart=None, onend=None, datamgr=None,
                 time=time.time):
        self.id = id
        self._datamgr = datamgr
        self._container = {}
        self._created = self._last_accessed = time()
        self._onend = onend
        self._onend_called = 0
        self._onstart = onstart
        self._onstart_called = 0

    def getDataMgr(self):
        return self._datamgr
        
    def onStart(self, sdm):
        if self._onstart and not self._onstart_called:
            onstart = self._onstart.__of__(sdm)
            onstart(self)
            self._onstart_called = 1
        
    def onEnd(self, sdm):
        if self._onend and not self._onend_called:
            onend = self._onend.__of__(sdm)
            onend(self)
            self._onend_called = 1

    def __getitem__(self, k):
        return self._container[k]

    def __setitem__(self, k, v):
        # if the key or value is a persistent instance,
        # set up its _p_jar immediately
        if hasattr(v, '_p_jar') and v._p_jar is None:
            v._p_jar = self._p_jar
            v._p_changed = 1
        if hasattr(k, '_p_jar') and k._p_jar is None:
            k._p_jar = self._p_jar
            k._p_changed = 1
        self._container[k] = v
        self._p_changed = 1

    set = __setitem__

    def __len__(self):
        return len(self._container)
    
    def keys(self):
        return self._container.keys()

    def items(self):
        return self._container.items()

    def values(self):
        return self._container.values()

    def get(self, k, default=None):
        return self._container.get(k, default)
        
    def has_key(self, k, marker=_marker):
        if self._container.get(k, marker) is not _marker: return 1

    def delete(self, k):
        del self._container[k]
        self._p_changed = 1
        
    def getId(self):
        return self.id

    def clear(self):
        self._container.clear()
        self._p_changed = 1

    def update(self, d):
        for k in d.keys():
            self[k] = d[k]

    def copy(self):
        return self._container.copy()

    def getCreated(self):
        return self._created

    def getLastAccessed(self):
        return self._last_accessed

    def setLastAccessed(self, time=time.time):
        self._last_accessed = time()

    def invalidate(self):
        if hasattr(self, 'aq_parent'):
            self.onEnd(self.aq_parent)
        self._invalid = None

    def __repr__(self):
        return `self._container`

    def _p_independent(self):
        # My state doesn't depend on or materially effect the state of
        # other objects.
        return 1

Globals.default__class_init__(SessionData)


