##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################
"""

Session data container

$Id: SessionDataContainer.py,v 1.38 2001/03/30 01:21:13 chrism Exp $
"""

__version__='$Revision: 1.38 $'[11:-2]

import Globals
from Globals import HTMLFile, MessageDialog
from SessionStorage import SessionStorage
from ZODB.DB import DB
from Persistence import Persistent, PersistentMapping
from Acquisition import Implicit
from OFS.SimpleItem import SimpleItem
import threading, os
lock = threading.Lock()
del threading

try:
    # Zope 2.3.1 and up
    from BTrees import OOBTree
    ctype = OOBTree.OOBTree
    del OOBTree
    from AutoExpireMapping import AutoExpireMapping
    SessionContainer = AutoExpireMapping
    del AutoExpireMapping

except ImportError:
    # Zopes 2.3.0 and under
    import BTree
    ctype = BTree.BTree
    del BTree
    from SessionFanout import SessionFanout
    SessionContainer = SessionFanout
    del SessionFanout

try:
    from SessioningInterfaces import SessionDataContainerInterface
except:
    # pre-2.3 Zopes don't have Interfaces module
    class SessionDataContainerInterface:
        pass    

# dbs is mapping of path to in-memory ZODB dbs which hold session data
# containers
dbs = {}

constructSessionDataContainerForm = HTMLFile('addExtDataContainer',
                                             globals())

def constructSessionDataContainer(self, id, title='',timeout_mins=20,
                                  REQUEST=None):
    """ """
    ob = SessionDataContainer(id, title, timeout_mins)
    self._setObject(id, ob)
    if REQUEST is not None:
        return self.manage_main(self, REQUEST, update_menu=1)

class SessionDataContainerErr(Exception): pass

class SessionDataContainer(SimpleItem, SessionContainer):
    """ """
    meta_type = 'Session Data Container'

    icon='misc_/CoreSessionTracking/datacontainer.gif'

    __implements__ = SessionDataContainerInterface

    manage_options=(
        {'label': 'Manage', 'action':'manage_container'},
        {'label': 'Security', 'action':'manage_access'},
        {'label': 'Advanced', 'action':'manage_advanced'}
        )
    
    __ac_permissions__=(
        ('Access session data',
         ['get', 'set', 'has_key', 'getLen'],
         ['Anonymous', 'Manager']),
        ('View management screens',
         ['manage_container', 'manage_access', 'manage_advanced',
          'getTimeoutMinutes'],
         ['Manager']),
        ('Manage Session Data Container',
         ['setTimeoutMinutes', 'setTitle','manage_changeSessionDataContainer',
          'manage_importSessionData', 'manage_exportSessionData'],
         ['Manager']),
        )
    
    def __init__(self, id, title='', timeout_mins=20, ctype=ctype,
                 onend='onEnd', err_margin=.20):
        SessionDataContainer.inheritedAttribute('__init__')(
            self, timeout_mins, ctype, onend, err_margin
            )
        self.id = id
        self.setTitle(title)
        self.setTimeoutMinutes(timeout_mins)
        
    def set(self, k, v):
        """ """
        self[k] = v

    def __len__(self):
        return self.getLen()

    manage_container = HTMLFile('manageExtDataContainer',globals())

    manage_advanced = HTMLFile('manageImpExpSessionData', globals())

    def manage_changeSessionDataContainer(
        self, title, timeout_mins, REQUEST=None
        ):
        """ """
        self.setTitle(title)
        self.setTimeoutMinutes(timeout_mins)
        if REQUEST is not None:
            return self.manage_container(self, REQUEST)

    def manage_exportSessionData(self, REQUEST=None):
        """ """
        f = os.path.join(Globals.data_dir, 'sessiondata.zexp')
        self.c = PersistentMapping()
        for k, v in self.items():
            self.c[k] = v
        get_transaction().commit()
        self.c._p_jar.exportFile(self.c._p_oid, f)
        del self.c
        if REQUEST is not None:
            return MessageDialog(title='Session data exported',
                                 message='Session data exported to %s' % f,
                                 action='manage_container')
        
    def manage_importSessionData(self, REQUEST=None):
        """ """
        f = os.path.join(Globals.data_dir, 'sessiondata.zexp')
        conn = self._p_jar
        ob = conn.importFile(f)
        for k, v in ob.items():
            self[k] = v
        if REQUEST is not None:
            return MessageDialog(title='Session data imported',
                                 message='Session data imported from %s' %f,
                                 action='manage_container')
            

    def setTitle(self, title):
        if not title:
            self.title = ''
        else:
            self.title = str(title)

    def getTimeoutMinutes(self):
        """ """
        return SessionDataContainer.inheritedAttribute('getTimeoutMinutes')(
            self
            )

    def setTimeoutMinutes(self, timeout_mins):
        """ """
        if timeout_mins != self.getTimeoutMinutes():
            # order of calling is important!
            self._setTimeout(timeout_mins)
            self._reset()

    def getSessionDataContainer(self):
        return self

class SessionDataContainerMount(Persistent, Implicit):
    """
    Singleton in-RAM SessionStorage mounted database which returns a
    SessionDataContainer as its single interesting object
    """

    def __init__(self):
        self._clear()
        
    def __setstate__(self, state):
        self._clear()

    def _clear(self):
        self._v_db = None
        self._v_conn = None
        self._v_parentpath = None
        self._v_dc_timeout = None

    def __of__(self, parent):
        parentpath = self._v_parentpath
        if parentpath is None:
            parentpath = self._v_parentpath = parent.getPhysicalPath()
        timeout = self._v_dc_timeout
        if timeout is None:
            timeout = self._v_timeout = parent.internal_dc_timeout
        return self._getContainer(parent, timeout).__of__(parent)
        
    def _getDB(self, parentpath):
        self._v_db = dbs.get(parentpath, None)
        if self._v_db is None:
            lock.acquire()
            try:
                s = SessionStorage()
                dbs[parentpath] = self._v_db = DB(s)
            finally:
                lock.release()
        return self._v_db
        
    def _getConnection(self, parent):
        db = self._v_db
        if db is None:
            db = self._getDB(self._v_parentpath)
        conn = self._v_conn
        if conn is None:
            conn = self._v_conn = db.open() # no versioning
            # if we don't have a jar yet, we steal our parent's
            jar = self._p_jar or parent._p_jar
            jar.onCloseCallback(ConnectionCloser(self, conn))
        return conn
    
    def _getContainer(self, parent, timeout):
        conn = self._getConnection(parent)
        root = conn.root()
        container = root.get('container', None)
        if container is None:
            root['container'] = SessionDataContainer(
                id='internal', timeout_mins=timeout
                )
        return root['container']
        
    def __del__(self):
        lock.acquire()
        try:
            try:
                dbs[self._v_parentpath] = None # in case of circ refs
                del dbs[self._v_parentpath]
            except:
                pass
        finally:
            lock.release()

class ConnectionCloser:
    def __init__(self, mount, conn):
        self.mount = mount
        self.conn = conn
        
    def __call__(self):
        """ This is called by the 'parent' connection's onCloseCallback
        handler, so that our 'spawned' connection will be closed when
        our parent connection is closed. """
        self.mount._v_conn = None
        self.conn.close()
            
Globals.default__class_init__(SessionDataContainer)
Globals.default__class_init__(SessionDataContainerMount)
