##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################
"""
Core session tracking Session Data Manager and SessionData.

$Id: SessionDataManager.py,v 1.49 2001/04/01 01:25:09 chrism Exp $
"""

__version__='$Revision: 1.49 $'[11:-2]

import re, time, string, sys
import Globals
from Globals import HTMLFile
from OFS.SimpleItem import Item
from Acquisition import Implicit, Explicit
from Persistence import Persistent
from AccessControl.Owned import Owned
from AccessControl.Role import RoleManager
from AccessControl import getSecurityManager
from AccessControl.SecurityManagement import newSecurityManager,\
     noSecurityManager
from App.Management import Tabs
from SessionDataContainer import SessionDataContainerMount
from SessionData import SessionData
from zLOG import LOG, WARNING
try:
    from SessioningInterfaces import SessionDataManagerInterface
except:
    # pre-2.3 Zopes don't have an Interfaces module
    class SessionDataManagerInterface:
        pass

WRITE_EVERY = 30 # time in seconds to write last_accessed to data objects

bad_path_chars_in=re.compile('[^a-zA-Z0-9-_~\,\. \/]').search

class SessionDataManagerErr(Exception): pass

class InvalidSessionState(SessionDataManagerErr): pass

constructSessionDataManagerForm = HTMLFile('addDataManager', globals())

def constructSessionDataManager(self, id, title='', path=None,
                                timeout_mins=20, onstart=None,
                                onend=None, REQUEST=None):
    """ """
    ob = SessionDataManager(id, path, title, timeout_mins, onstart, onend)
    self._setObject(id, ob)
    if REQUEST is not None:
        return self.manage_main(self, REQUEST, update_menu=1)

class SessionDataManager(Item,Implicit,Persistent,RoleManager,Owned,Tabs):
    """ The Zope default session data manager implementation """

    meta_type = 'Session Data Manager'

    manage_options=(
        {'label': 'Settings', 'action':'manage_sessiondatamgr'},
        {'label': 'Internal Data Container',
         'action':'manage_internaldc'},
        {'label': 'Security', 'action':'manage_access'},
        {'label': 'Ownership', 'action':'manage_owner'},
        )

    __ac_permissions__=(
        ('Access contents information',
         ['encodeUrl', 'isTokenNew', 'isTokenFromCookie', 'isTokenFromForm',
          'getToken', 'hasToken', 'getTokenKey'],
         ['Manager', 'Anonymous']),
        ('Access session data',
         ['getSessionData', 'hasSessionData', 'getUserData', 'hasUserData'],
         ['Manager','Anonymous']),
        ('View management screens',
         ['manage_sessiondatamgr', 'manage_internaldc', 'getInternalDCLen',
          'getInternalDCTimeoutMins', 'getSessionDataContainerPath',
          'getOnStartPath', 'getOnEndPath'],
         ['Manager']),
        ('Change Session Data Manager',
         ['setSessionDataContainerPath', 'manage_changeSessionDataManager',
          'manage_changeInternalDC', 'setOnStartPath', 'setOnEndPath',
          'setSessionDataContainerPath', 'setTitle'],
         ['Manager']),
        )

    icon='misc_/CoreSessionTracking/datamgr.gif'

    __allow_access_to_unprotected_subobjects__ = 0

    __implements__ = SessionDataManagerInterface

    manage_sessiondatamgr = HTMLFile('manageDataManager', globals())

    manage_internaldc=HTMLFile('manageIntDataContainer',globals())
    
    def __init__(self, id, path=None, title='', timeout_mins=20,
                 onstart=None, onend=None):
        self.id = id
        self.setTitle(title)
        self.setSessionDataContainerPath(path)
        self.setInternalDCTimeoutMins(timeout_mins)
        self.setOnStartPath(onstart)
        self.setOnEndPath(onend)

    def setInternalDCTimeoutMins(self, timeout_mins):
        if type(timeout_mins) is not type(1):
            raise SessionDataManagerErr, (
                "Internal data container timeout must be integer: %s" %
                timeout_mins
                )
        self.internal_dc_timeout = timeout_mins

    def manage_afterAdd(self, item, container):
        """ """
        self._sdc = SessionDataContainerMount()
        self._sdc.setTimeoutMinutes(self.internal_dc_timeout)
        
    def manage_changeInternalDC(self, timeout_mins, REQUEST=None):
        """ """
        self.setInternalDCTimeoutMins(timeout_mins)
        self._sdc.setTimeoutMinutes(timeout_mins)
        if REQUEST is not None:
            return self.manage_internaldc(self, REQUEST)

    def getInternalDCLen(self):
        """ """
        return len(self._sdc)

    def getInternalDCTimeoutMins(self):
        """ """
        return self.internal_dc_timeout

    def manage_changeSessionDataManager(self, title, path=None,
                                        onstart=None, onend=None,
                                        REQUEST=None):
        """ """
        self.setTitle(title)
        self.setSessionDataContainerPath(path)
        self.setOnStartPath(onstart)
        self.setOnEndPath(onend)
        if REQUEST is not None:
            return self.manage_sessiondatamgr(self, REQUEST)

    def getSessionData(self, create=1):
        """ """
        key = self.getToken(create=create)
        data = self._getSessionDataObject(key)
        if data is None and create:
            return self._createSessionDataObject(key)
        else:
            return data
        # fall off the end and return None if create=0 or invalid data

    def hasSessionData(self):
        """ """
        if self.getSessionData(create=0) is not None: return 1

    def getSessionIdManager(self):
        """ """
        mgr = getattr(self, 'session_id_mgr', None)
        if mgr is None:
            raise SessionDataManagerErr,(
                'No session id manager could be found.'
                )
        return mgr
    
    def encodeUrl(self, url, create=1):
        """ """
        return self.getSessionIdManager().encodeUrl(url, create)

    def getToken(self, create=1):
        """ """
        return self.getSessionIdManager().getToken(create)
        
    def hasToken(self):
        """ """
        return self.getSessionIdManager().hasToken()

    def getTokenKey(self):
        """ """
        return self.getSessionIdManager().getTokenKey()

    def isTokenNew(self):
        """ """
        return self.getSessionIdManager().isTokenNew()

    def isTokenFromCookie(self):
        """ """
        return self.getSessionIdManager().isTokenFromCookie()
    
    def isTokenFromForm(self):
        """ """
        return self.getSessionIdManager().isTokenFromForm()

    def setTitle(self, title):
        """ """
        if not title:
            self.title = ''
        else:
            self.title = str(title)

    def setSessionDataContainerPath(self, path=None):
        """ """
        if not path:
            self.obpath = None # use internal data container
        elif type(path) is type(''):
            if bad_path_chars_in(path):
                raise SessionDataManagerErr, (
                    'Container path contains characters invalid in a Zope '
                    'object path'
                    )
            self.obpath = string.split(path, '/')
        elif type(path) in (type([]), type(())):
            self.obpath = list(path) # sequence
        else:
            raise SessionDataManagerErr, ('Bad path value %s' % path)
            
    def getSessionDataContainerPath(self):
        """ """
        if self.obpath is not None:
            return string.join(self.obpath, '/')
        return '' # blank string represents internal data container
    
    def _createSessionDataObject(self, key, SessionData=SessionData):
        """ This must never fail """
        if not hasattr(self, '_v_path'):
            self._v_path = self.getPhysicalPath()
        container = self._getSessionDataContainer()
        ob = SessionData(id=key,
                         onstart=self._getOnStartFunction(),
                         onend=self._getOnEndFunction(),
                         datamgr=self._v_path)
        container.set(key, ob)
        ob.onStart(self)
        return ob.__of__(self)

    def _getSessionDataObject(self, key, time=time.time,
                              WRITE_EVERY=WRITE_EVERY,
                              sdType=type(SessionData(1))):
        """ returns None if invalid or doesn't exist """
        container = self._getSessionDataContainer()
        ob = container.get(key)
        if ob is not None:
            bare = getattr(ob, 'aq_base', ob)
            if type(bare) is sdType:
                if not hasattr(bare, '_invalid'):
                    now = time()
                    la = ob.getLastAccessed()
                    if (now - la) >= WRITE_EVERY:
                        ob.setLastAccessed()
                    return ob.__of__(self)

    def _getSessionDataContainer(self):
        """ Do not cache the results of this call.  Doing so breaks the
        transactions for mounted storages. """
        if self.obpath is not None:
            # return an external data container
            try:
                # This should arguably use restrictedTraverse, but it
                # currently fails for mounted storages.  This might
                # be construed as a security hole, albeit a minor one.
                # unrestrictedTraverse is also much faster.
                return self.unrestrictedTraverse(self.obpath)
            except:
                raise SessionDataManagerErr, (
                    "External session data container '%s' not found." %
                    string.join(self.obpath,'/')
                    )
        else:
            # otherwise return a ram-based internal data container
            return self._sdc.__of__(self)

    def _setOnStartOrEndPath(self, item, name):
        """ Accepts as 'item' the None value or a string or a list/tuple
        which represents the absolute or relative path to a Zope function
        object (e.g. external method, Python method, etc) and associates
        it with an event handler. The fn is looked up via
        restrictedTraverse at execution time and is executed in the
        context of the owner of the session data manager.

        Onstart or onend functions should accept a single argument which is
        a mapping object (the session data object being started or
        ended).

        If item is None, the session tracking machinery essentially
        foregoes attempting to execute an event."""
        if not item:
            setattr(self, name, None) # e.g. self._onstart = None
        elif type(item) is type(''):
            if bad_path_chars_in(item):
                raise SessionDataManagerErr, (
                    '%s path contains characters invalid in '
                    'a Zope object path' % name
                    )
            setattr(self, name, string.split(item, '/'))
        elif type(item) in (type([]), type(())):
            setattr(self, name, list(item))
        else:
            raise SessionDataManagerErr, ('Bad %s value %s' % (name, item))

    def setOnStartPath(self, path):
        """ """
        self._setOnStartOrEndPath(path, '_onstart')
        
    def setOnEndPath(self, path):
        """ """
        self._setOnStartOrEndPath(path, '_onend')

    def getOnStartPath(self):
        """ """
        if hasattr(self, '_onstart') and self._onstart is not None:
            return string.join(self._onstart, '/')
        return ''

    def getOnEndPath(self):
        """ """
        if hasattr(self, '_onend') and self._onend is not None:
            return string.join(self._onend, '/')
        return ''
    
    def _getOnStartOrEndFunction(self, name):
        f = getattr(self, name)
        if f:
            try:
                return EventWrapper(f)
            except:
                LOG('Session Tracking', 0, 'Failed to initialize an %s '
                    'function ' % name, '', sys.exc_info())
                
    def _getOnStartFunction(self):
        return self._getOnStartOrEndFunction('_onstart')

    def _getOnEndFunction(self):
        return self._getOnStartOrEndFunction('_onend')


class EventWrapper(Item, Persistent, Implicit):
    """
    a class encapsulating a function, optionally in the context of a specific
    user at execution time.  This is necessary as a "soft link" to store
    references to functions across ZODB databases.
    """
    def __init__(self, fpath, ufolderpath=None, uid=None):
        self.f = fpath
        self.ufolder = ufolderpath
        self.uid = uid
        
    def __call__(self, sdo):
        """ This should never fail with an exception """
        user_changed = 0
        try:
            try:
                try:
                    f = self.unrestrictedTraverse(self.f)
                except:
                    fpath = string.join(self.f, '/')
                    raise SessionDataManagerErr, (
                        'the session event function named %s could not be '
                        'found in the ZODB' % fpath
                        )
                if self.ufolder is not None:
                    user = getSecurityManager().getUser()
                    ufolder = self.unrestrictedTraverse(self.ufolder)
                    u = ufolder.getUser(self.uid)
                    u = getattr(u, 'aq_base', u)
                    u = u.__of__(ufolder)
                    user_changed = 1
                    newSecurityManager(None, u) # switch to saved user
                try:
                    f(sdo)
                except:
                    if hasattr(f, 'id'):
                        if callable(f.id): id = f.id()
                        else: id = f.id
                    else:
                        id = None
                    LOG('Session Tracking',WARNING,'session event failed',
                        'The call to function %s failed.  Traceback:\n'
                        % id, sys.exc_info())
            except:
                LOG('Session Tracking', WARNING, 'session event failed',
                    'Uncaught error\n', sys.exc_info())
        finally:
            if user_changed:
                # switch back to the original user
                newSecurityManager(None, user)


Globals.default__class_init__(SessionDataManager)
Globals.default__class_init__(EventWrapper)
