##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################
"""

Session fanout in lieu of a new BTree implementation.  This is lame and should
definitely go away after a new BTree implementation is finished.

$Id: SessionFanout.py,v 1.5 2001/03/30 01:21:13 chrism Exp $
"""

__version__ = '$Revision: 1.5 $'[11:-2]

from AutoExpireMapping import PersistentAutoExpireMapping
from Persistence import Persistent
import Acquisition
_marker = []
allowed_hashes = map(lambda x: "%02i" % x, range(0, 100))

class SessionFanout:
    """
    Pure mixin.
    
    A mapping which stores its items in a number of separate data structures
    in order to reduce the probability of conflict in high-write situations.
    Keys must be strings or integers.  If keys are strings, the first
    two characters of the key must be digits.
    """
    def __init__(self, timeout_mins=20, ctype=None, onend='onEnd',
                 err_margin=.20):
        self._initleaves(timeout_mins, ctype, onend, err_margin)
        self._timeout_secs = timeout_mins * 60
        self._ctype = ctype
        self._onend = onend
        self._err_margin = err_margin
        
    def _setTimeout(self, timeout_mins):
        for leaf in self._root:
            leaf = leaf.__of__(self)
            leaf._setTimeout(timeout_mins)
        self._timeout_secs = timeout_mins * 60
        
    def getTimeoutMinutes(self):
        """ """
        return self._timeout_secs / 60

    def _reset(self):
        for leaf in self._root:
            leaf = leaf.__of__(self)
            leaf._reset()

    def _initleaves(self, timeout_mins, ctype, onend, err_margin):
        self._root = []
        for x in range(0,100):
            m=PersistentAutoExpireMapping(timeout_mins,ctype,onend,err_margin)
            self._root.append(m)
            
    def _getleaf(self, key, allowed_hashes=allowed_hashes):
        if key is not None:
            if type(key) is not type(''):
                key = `key`
            if key[:2] in allowed_hashes:
                lkey = int(key[:2])
                leaf = self._root[lkey]
                return leaf.__of__(self)
            raise 'First two elements in key must be digits'
        else:
            return {}
    
    def __delitem__(self, key):
        leaf = self._getleaf(key)
        del leaf[key]

    def __getitem__(self, key):
        leaf = self._getleaf(key)
        return leaf[key]
        
    def __len__(self):
        l = 0
        for leaf in self._root:
            leaf = leaf.__of__(self)
            l = l + len(leaf)
        return l
        
    getLen = __len__

    # something is screwing up the inheritance of __len__ here, getLen
    # is a workaround

    def __setitem__(self, key, v):
        leaf = self._getleaf(key)
        leaf[key] = v
        
    def copy(self):
        raise 'Unsupported'

    def get(self, key, default=_marker):
        leaf = self._getleaf(key)
        if default is _marker:
            return leaf.get(key)
        else:
            return leaf.get(key, default)

    def has_key(self, key):
        leaf = self._getleaf(key)
        return leaf.has_key(key)
            
    def items(self):
        items = []
        for leaf in self._root:
            leaf = leaf.__of__(self)
            leafitems = map(lambda k,d=leaf: (k, d[k]), leaf.keys())
            items.extend(leafitems)
        items.sort()
        return items

    def keys(self):
        keys = []
        for leaf in self._root:
            leaf = leaf.__of__(self)
            keys.extend(map(lambda k: k, leaf.keys()))
        keys.sort()
        return keys
    
    def update(self, b):
        raise 'Unsupported'

    def values(self):
        values = []
        for leaf in self._root:
            leaf = leaf.__of__(self)
            values.extend(map(lambda k: k, leaf.values()))
        values.sort()
        return values

