##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################
"""
Session Id Managers keep track of client browser keys.

$Id: SessionIdManager.py,v 1.32 2001/04/04 00:55:11 chrism Exp $
"""

__version__='$Revision: 1.32 $'[11:-2]

import Globals
from Persistence import Persistent
from ZODB import TimeStamp
from Acquisition import Implicit
from AccessControl.Owned import Owned
from AccessControl.Role import RoleManager
from App.Management import Tabs
from Globals import HTMLFile
from OFS.SimpleItem import Item
from OFS.ObjectManager import UNIQUE

try:
    from SessioningInterfaces import SessionIdManagerInterface
except:
    # pre-2.3 Zopes don't have an Interfaces module
    class SessionIdManagerInterface:
        pass

import os, time, random, string, binascii, sys, re

b64_trans = string.maketrans('+/', '-.')
b64_untrans = string.maketrans('-.', '+/')

badtokenkeycharsin = re.compile('[\?&;, ]').search
badcookiecharsin = re.compile('[;, ]').search
twodotsin = re.compile('(\w*\.){2,}').search

_marker = []

constructSessionIdManagerForm = HTMLFile('addIdManager',globals())

def constructSessionIdManager(
    self, id, title='', tokenkey='_ZopeId', cookiepri=1, formpri=2,
    urlpri=0, cookiepath='/', cookiedomain='', cookielifedays=0,
    cookiesecure=0, rewritemode='querystring', REQUEST=None
    ):
    """ """
    # flip dictionary and take what's not 0 (god I hate HTML)
    d = {}
    for k,v in {'url':urlpri, 'form':formpri, 'cookies':cookiepri}.items():
        if v: d[v] = k
    ob = SessionIdManager(id, title, tokenkey, d, cookiepath,
                          cookiedomain, cookielifedays, cookiesecure,
                          rewritemode)
    self._setObject(id, ob)
    ob = self._getOb(id)
    if REQUEST is not None:
        return self.manage_main(self, REQUEST, update_menu=1)
    
class SessionIdManagerErr(Exception): pass

class SessionIdManager(Item, Persistent, Implicit, RoleManager, Owned, Tabs):
    """ session id management class """

    meta_type = 'Session Id Manager'

    manage_options=(
        {'label': 'Settings', 'action':'manage_sessionidmgr'},
        {'label': 'Security', 'action':'manage_access'},
        {'label': 'Ownership', 'action':'manage_owner'}
        )

    __ac_permissions__ = (
        ('Change Session Id Manager',
         ['turnOn', 'turnOff', 'setTokenKey', 'setTokenKeyNamespaces',
          'setCookieSecure', 'setCookieDomain', 'setCookieLifeDays',
          'setCookiePath', 'manage_changeSessionIdManager'],
         ['Manager']),
        ('View management screens',
         ['manage_sessionidmgr'],
         ['Manager']),
        ('Access contents information',
         ['getAllTokenKeyNamespaces', 'getCookieSecure',
          'getCookieDomain', 'getCookiePath',
          'getCookieLifeDays', 'getTokenKeyNamespaces',
          'getTokenKey', 'getToken', 'isTokenFromForm', 'isTokenFromCookie',
          'isTokenNew' 'hasToken', 'isOn', 'encodeUrl'],
         ['Anonymous', 'Manager'])
        )

    __allow_access_to_unprotected_subobjects__ = 0

    __replaceable__ = UNIQUE # singleton for now

    __implements__ = SessionIdManagerInterface

    icon = 'misc_/CoreSessionTracking/idmgr.gif'

    def __init__(
        self, id, title='', tokenkey='_ZopeId',
        tokenkeynamespaces={1:'cookies',2:'form'}, cookiepath=('/'),
        cookiedomain='', cookielifedays=0, cookiesecure=0, on=1
        ):

        self.id = id
        self.title = title
        self.setTokenKey(tokenkey)
        self.setTokenKeyNamespaces(tokenkeynamespaces)
        self.setCookiePath(cookiepath)
        self.setCookieDomain(cookiedomain)
        self.setCookieLifeDays(cookielifedays)
        self.setCookieSecure(cookiesecure)
        if on:
            self.turnOn()
        else:
            self.turnOff()

    # delegating methods follow
    # don't forget to change the name of the method in
    # delegation if you change a delegating method name
    
    def hasToken(self):
        """ Returns true if there is a current session token, but does
        not create a session token for the current request if one doesn't
        already exist """
        if not self.on:
            return self._delegateToParent('hasToken')
        if self.getToken(create=0): return 1
                
    def getToken(self, create=1):
        """
        Examines the request and hands back session token value or
        None if no token exists.  If there is no session token
        and if 'create' is true, create one.  If cookies are are
        an allowable id key namespace and create is true, set one.  Stuff
        the token and the namespace it was found in into the REQUEST object
        for further reference during this request.  Delegate this call to
        a parent if we're turned off.
        """
        if not self.on:
            return self._delegateToParent('getToken', create)
        REQUEST = self.REQUEST
        # let's see if token has already been attached to request
        token = getattr(REQUEST, 'session_token_', None)
        if token is not None:
            # it's already set in this request so we can just return it
            # if it's well-formed
            if not self._isAWellFormedToken(token):
                # somebody screwed with the REQUEST instance during
                # this request.
                raise SessionIdManagerErr, (
                    'Ill-formed token in REQUEST.session_token_:  %s' % token
                    )
            return token
        # fall through & ck id key namespaces if token is not in request.
        tk = self.token_key
        ns = self.token_key_namespaces
        for name in ns:
            token = getattr(REQUEST, name).get(tk, None)
            if token is not None:
                # hey, we got a token!
                if self._isAWellFormedToken(token):
                    # token is not "plain old broken"
                    REQUEST.session_token_ = token
                    REQUEST.session_token_ns_ = ns
                    return token
        # fall through if token is invalid or not in key namespaces
        if create:
            # create a brand new token
            token = self._getNewToken()
            if 'cookies' in ns:
                self._setCookie(token, REQUEST)
            REQUEST.session_token_ = token
            REQUEST.session_token_ns_ = None
            return token
        # implies a return of None if:
        # (not create=1) and (invalid or ((not in req) and (not in ns)))


    def isTokenFromCookie(self):
        """ returns true if session token is from REQUEST.cookies """
        if not self.on:
            return self._delegateToParent('isTokenFromCookie')
        v = getattr(self.REQUEST,'session_token_ns_', _marker)
        if v is _marker:
            raise SessionIdManagerErr, 'There is no current session token.'
        elif v == 'cookies':
            return 1

    def isTokenFromForm(self):
        """ returns true if session token is from REQUEST.form """
        if not self.on:
            return self._delegateToParent('isTokenFromForm')
        v = getattr(self.REQUEST,'session_token_ns_', _marker)
        if v is _marker:
            raise SessionIdManagerErr, 'There is no current session token.'
        elif v == 'form':
            return 1

    def isTokenNew(self):
        """
        returns true if session token is 'new', meaning the token exists
        but it has not yet been acknowledged by the client (the client
        hasn't sent it back to us in a cookie or in a formvar).
        """
        if not self.on:
            return self._delegateToParent('isTokenNew')
        v = getattr(self.REQUEST, 'session_token_ns_', _marker)
        if v is _marker:
            raise SessionIdManagerErr, 'There is no current session token.'
        # ns will be None if new, negating None below returns 1, which
        # would indicate that it's new on this request
        return not v
    
    def encodeUrl(self, url, create=1):
        """
        encode a URL with the session key as a postfixed query string
        element
        """
        if not self.on:
            return self._delegateToParent('encodeUrl', url)
        token = self.getToken(create)
        if token is None:
            raise SessionIdManagerErr, 'There is no current session token.'
        key = self.getTokenKey()
        if '?' in url:
            return '%s&%s=%s' % (url, key, token)
        else:
            return '%s?%s=%s' % (url, key, token)

    # non-delegating methods follow
    
    manage_sessionidmgr = HTMLFile('manageIdManager', globals())

    def manage_changeSessionIdManager(
        self, title='', tokenkey='_ZopeId', cookiepri=1, formpri=2,
        cookiepath='/', cookiedomain='', cookielifedays=0, cookiesecure=0,
        on=0, REQUEST=None
        ):
        """ """
        d = {}
        for k,v in {'cookies':cookiepri, 'form':formpri}.items():
            if v: d[v] = k # I hate HTML
        self.title = title
        self.setTokenKey(tokenkey)
        self.setTokenKeyNamespaces(d)
        self.setCookiePath(cookiepath)
        self.setCookieDomain(cookiedomain)
        self.setCookieLifeDays(cookielifedays)
        self.setCookieSecure(cookiesecure)
        if on:
            self.turnOn()
        else:
            self.turnOff()
        if REQUEST is not None:
            return self.manage_sessionidmgr(self, REQUEST)

    def setTokenKey(self, k):
        """ sets session token key string """
        if not (type(k) is type('') and k and not badtokenkeycharsin(k)):
            raise SessionIdManagerErr, 'Bad id key string %s' % repr(k)
        self.token_key = k

    def getTokenKey(self):
        """ """
        return self.token_key

    def setTokenKeyNamespaces(self,namespacesd={1:'cookies',2:'form'}):
        """
        accepts dictionary e.g. {1: 'cookies', 2: 'form'} as token
        id key allowable namespaces and lookup ordering priority
        where key is 'priority' with 1 being highest.
        """
        allowed = self.getAllTokenKeyNamespaces()
        for name in namespacesd.values():
            if name not in allowed:
                raise SessionIdManagerErr, (
                    'Bad id key namespace %s' % repr(name)
                    )
        self.token_key_namespaces = []
        nskeys = namespacesd.keys()
        nskeys.sort()
        for priority in nskeys:
            self.token_key_namespaces.append(namespacesd[priority])

    def getTokenKeyNamespaces(self):
        """ """
        d = {}
        i = 1
        for name in self.token_key_namespaces:
            d[i] = name
            i = i + 1
        return d

    def setCookiePath(self, path=''):
        """ sets cookie 'path' element for id cookie """
        if not (type(path) is type('') and not badcookiecharsin(path)):
            raise SessionIdManagerErr, 'Bad cookie path %s' % repr(path)
        self.cookie_path = path
    
    def getCookiePath(self):
        """ """
        return self.cookie_path

    def setCookieLifeDays(self, days):
        """ offset for id cookie 'expires' element """
        if type(days) not in (type(1), type(1.0)):
            raise SessionIdManagerErr,(
                'Bad cookie lifetime in days %s (requires integer value)'
                % repr(days)
                )
        self.cookie_life_days = int(days)

    def getCookieLifeDays(self):
        """ """
        return self.cookie_life_days

    def setCookieDomain(self, domain):
        """ sets cookie 'domain' element for id cookie """
        if type(domain) is not type(''):
            raise SessionIdManagerErr, (
                'Cookie domain must be string: %s' % repr(domain)
                )
        if not domain:
            self.cookie_domain = ''
            return
        if not twodotsin(domain):
            raise SessionIdManagerErr, (
                'Cookie domain must contain at least two dots (e.g. '
                '".zope.org" or "www.zope.org") or it must be left blank. : '
                '%s' % `domain`
                )
        if badcookiecharsin(domain):
            raise SessionIdManagerErr, (
                'Bad characters in cookie domain %s' % `domain`
                )
        self.cookie_domain = domain

    def getCookieDomain(self):
        """ """
        return self.cookie_domain

    def setCookieSecure(self, secure):
        """ sets cookie 'secure' element for id cookie """
        self.cookie_secure = not not secure

    def getCookieSecure(self):
        """ """
        return self.cookie_secure

    def getAllTokenKeyNamespaces(self):
        """
        These are the REQUEST namespaces searched when looking for an
        id key value.
        """
        return ('form', 'cookies')

    def turnOn(self):
        """ """
        self.on = 1

    def turnOff(self):
        """ """
        self.on = 0

    def isOn(self):
        """ """
        return self.on

    # non-interface methods follow

    def _getNewToken(self, randint=random.randint, maxint=99999999):
        """ Returns 19-character string session token
        'AAAAAAAABBBBBBBB'
        where:

        A == leading-0-padded 8-char string-rep'd random integer
        B == modified base64-encoded 11-char timestamp
        
        To be URL-compatible, base64 encoding is modified as follows:
          '=' end-padding is stripped off
          '+' is translated to '-'
          '/' is translated to '.'
        """
        return '%08i%s' % (randint(0, maxint-1), self._getB64TStamp())
    
    def _delegateToParent(self, *arg, **kw):
        fn = arg[0]
        rest = arg[1:]
        try:
            parent_sessidmgr=getattr(self.aq_parent, self.id)
            parent_fn = getattr(parent_sessidmgr, fn)
        except AttributeError:
            raise SessionIdManagerErr, 'Session id management disabled'
        return apply(parent_fn, rest, kw)

    def _setCookie(
        self, token, REQUEST, now=time.time, strftime=time.strftime,
        gmtime=time.gmtime
        ):
        """ """
        expires = None
        if self.cookie_life_days:
            expires = now() + self.cookie_life_days * 86400
            # Wdy, DD-Mon-YYYY HH:MM:SS GMT
            expires = strftime('%a %d-%b-%Y %H:%M:%S GMT',gmtime(expires))
        d = {'domain':self.cookie_domain,'path':self.cookie_path,
             'secure':self.cookie_secure,'expires':expires}

        if self.cookie_secure:
            URL1 = REQUEST.get('URL1', None)
            if URL1 is None:
                return # should we raise an exception?
            if string.split(URL1,':')[0] != 'https':
                return # should we raise an exception?
         
        cookies = REQUEST.RESPONSE.cookies
        cookie = cookies[self.token_key]= {}
        for k,v in d.items():
            if v:
                cookie[k] = v #only stuff things with true values
        cookie['value'] = token
        
    def _getB64TStamp(
        self, b2a=binascii.b2a_base64,gmtime=time.gmtime, time=time.time,
        b64_trans=b64_trans, split=string.split,
        TimeStamp=TimeStamp.TimeStamp, translate=string.translate
        ):
        t=time()
        ts=split(b2a(`apply(TimeStamp,(gmtime(t)[:5]+(t%60,)))`)[:-1],'=')[0]
        return translate(ts, b64_trans)

    def _getB64TStampToInt(
        self, ts, TimeStamp=TimeStamp.TimeStamp, b64_untrans=b64_untrans,
        a2b=binascii.a2b_base64, translate=string.translate
        ):
        return TimeStamp(a2b(translate(ts+'=',b64_untrans))).timeTime()

    def _getTokenPieces(self, token):
        """ returns session token parts in a tuple consisting of rand_id,
        timestamp
        """
        return (token[:8], token[8:19])

    def _isAWellFormedToken(self, token, binerr=binascii.Error,
                            timestamperr=TimeStamp.error):
        try:
            rnd, ts = self._getTokenPieces(token)
            int(rnd)
            self._getB64TStampToInt(ts)
            return token
        except (TypeError, ValueError, AttributeError, IndexError, binerr,
                timestamperr):
            return None

    def _setId(self, id):
        if id != self.id:
            raise Globals.MessageDialog(
                title='Cannot rename',
                message='You cannot rename a session id manager, sorry!',
                action ='./manage_main',)


Globals.default__class_init__(SessionIdManager)












