##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################
"""
Core session tracking interfaces

$Id: SessioningInterfaces.py,v 1.4 2001/04/04 00:55:11 chrism Exp $
"""

import Interface

class SessionIdManagerInterface(Interface.Base):
    """
    Zope Session Id Manager interface.

    A Zope Session Id Manager is responsible for assigning ids to site
    visitors, and for servicing requests from Session Data Managers
    related to keys and tokens.
    """
    id = 'session_id_mgr'
    # id must always be 'session_id_mgr' in order to be found
    # via acquisition.

    def manage_changeSessionIdManager(
        self, title='', tokenkey='_ZopeId', cookiepri=1, formpri=2,
        cookiepath='/', cookiedomain='', cookielifedays=0, cookiesecure=0,
        on=0, REQUEST=None
        ):
        """
        Changes the session id manager settings.  See __init__ for
        parameter descriptions.

        Parameters:
        
        title -- Zope object title
        tokenkey -- key for token value in form/cookie namespaces
        cookiepri -- lookup priority of 'REQUEST.cookies' for tokenkey
        formpri -- lookup priority of 'REQUEST.form' for tokenkey 
        cookiepath -- cookie 'path' value (blank is none)
        cookiedomain -- cookie 'domain' value (blank is none)
        cookielifedays -- life in days of cookie (0 is nonpersistent)
        cookiesecure -- cookie 'secure' value
        on -- session id manager on/off toggle

        Permission required: Change Session Id Manager
        """

    def turnOn(self):
        """
        Turn the session id manager "on" (calls to it will not be delegated
        to an acquired session id manager).

        Permission required: Change Session Id Manager
        """

    def turnOff(self):
        """
        Turn the session id manager "off" (calls to it will be delegated to
        an acquired session id manager).

        Permission required: Change Session Id Manager
        """

    def isOn(self):
        """
        Returns true if session id manager is on.

        Permission required: Access contents information
        """

    def setTokenKey(self, k):
        """
        Set token key to k.

        Permission required:  Change Session Id Manager
        """

    def getTokenKey(self):
        """
        Return the token key (e.g. '_ZopeId').

        Permission required:  Access contents information
        """

    def setTokenKeyNamespaces(self, d):
        """
        Set token key namespace lookup ordering.  Token key namespace
        lookup order should be a dictionary, the dictionary should
        have integer keys, and its values should
        be either 'cookies' or 'form'.  E.g. {1:'cookies', 2:'form}.

        Permission required:  Change Session Id Manager
        """

    def getTokenKeyNamespaces(self):
        """
        Returns token key namespace lookup order as a dictionary (e.g.
        {1: 'cookies', 2: 'form']}, where the lookup order is implied by
        the key.

        Permission required:  Access contents information.
        """

    def setCookiePath(self, path=''):
        """
        Set session cookie 'path' element.

        Permission required: Change Session Id Manager
        """

    def getCookiePath(self):
        """
        Return session cookie 'path' element.

        Permission required:  Access contents information
        """

    def setCookieLifeDays(self, days):
        """
        Set lifetime of session cookie to 'days' number of days.

        Permission required:  Change Session Id Manager
        """

    def getCookieLifeDays(self):
        """
        Return lifetime of session cookie in days.

        Permission required:  Access contents information
        """

    def setCookieDomain(self, domain):
        """
        Set session cookie domain element.

        Permission required:  Change Session Id Manager
        """

    def getCookieDomain(self):
        """
        Return session cookie domain element.

        Permission required:  Access contents information.
        """

    def setCookieSecure(self, secure):
        """
        Set id manager to only send cookies across HTTPS connections if
        'secure' is passed in as a true value.  If 'secure' is true,
        the secure flag of all session cookies will be set as well.

        Permission required:  Change Session Id Manager
        """

    def getCookieSecure(self):
        """
        Return true if cookie secure for this id manager is true.

        Permission required:  Access contents information
        """



class SessionDataManagerInterface(Interface.Base):
    """
    Zope Session Data Manager interface.

    A Zope Session Data Manager is responsible for maintaining Session
    Data Objects, and for servicing requests from application code
    related to Session Data Objects.  It also communicates with a Session
    Id Manager to provide information about session tokens.
    """

    def manage_changeSessionDataManager(self, title, path=None,
                                        onstart=None, onend=None,
                                        REQUEST=None):
        """
        Change session data manager.

        title -- title of object.
        path -- string path to Session Data Container object (None
                indicates to use internal RAM-based data container)
        onstart -- physical path of onstart function
        onend -- physical path of onend function
        REQUEST -- if request not None, return an HTML response

        Permission required: Change Session Manager
        """
        
    def getSessionData(self, create=1):
        """
        Returns a Session Data Object associated with the current session
        token.  If there is no current token, and create is true, returns a
        new Session Data Object.  If there is no current token and create is
        false, returns None.

        Permission required: Access session data
        """

    def hasSessionData(self):
        """
        Returns true if a Session Data Object associated with the
        current session token is found in the Session Data Container.  Does
        not create a Session Data Object if one does not exist.

        Permission required: Access session data
        """

    def encodeUrl(self, url):
        """
        Encodes a provided URL with the current request's session token
        and returns the result.
        **Delegates to acquired session id manager**.

        Permission required: Access contents information

        Raises:  SessionIdManagerErr.  If there is no current session token.
        """

    def getTokenKey(self):
        """
        Returns a string with the name of the cookie/form variable which is
        used by the current session id manager as the name to look up when
        attempting to obtain the session token value.  For example, '_ZopeId'.
        **Delegates to acquired session id manager.**

        Permission required: Access contents information
        """

    def getToken(self, create=1):
        """
        If create=0, returns a the current session token or None if there
        is no session token associated with the current request.  If create=1,
        returns the current session token or a newly-created session token if
        there is no session token associated with the current request.  This
        method is useful in conjunction with getTokenKey if you wish to embed
        the token-key/token combination as a hidden value in a POST-based
        form.  The session token is opaque, has no business meaning, and its
        length, type, and composition are subject to change.
        **Delegates to acquired session id manager.**

        Permission required: Access contents information

        Raises:  SessionIdManagerErr.  If ill-formed session token
        is found in REQUEST.
        """

    def hasToken(self):
        """
        Returns true if there is a session token for this request.
        **Delegates to acquired session id manager.**

        Permission required: Access contents information
        """

    def isTokenNew(self):
        """
        Returns true if session token is 'new'.  A session token is 'new'
        when it is first created and the client has therefore not sent it
        back to the server in any request.  If there is no session token
        in the request, this method will raise a SessionIdManagerErr.
        **Delegates to acquired session id manager.**

        Permission required: Access contents information

        Raises:  SessionIdManagerErr.  If there is no current session token.
        """

    def isTokenFromForm(self):
        """
        Returns true if session token comes from a form variable (query
        string or post).   If there is no session token in the request,
        this method will raise a SessionIdManagerErr.
        **Delegates to acquired session id manager.**

        Permission required: Access contents information

        Raises:  SessionIdManagerErr.  If there is no current session token.
        """

    def isTokenFromCookie(self):
        """
        Returns true if session token comes from a cookie.   If there is no
        session token in the request, this method will raise a
        SessionIdManagerErr.
        **Delegates to acquired session id manager.**

        Permission required: Access contents information

        Raises:  SessionIdManagerErr.  If there is no current session token.
        """

    def setOnStartPath(self, path):
        """
        Sets the physical path which should be used to find a Zope
        object containing the session 'onStart' function.  The path
        must be a string, e.g. '/afolder/afunction'.

        """
        
    def setOnEndPath(self, path):
        """
        Sets the physical path which should be used to find a Zope
        object containing the session 'onEnd' function.  The path must
        be a string, e.g. '/afolder/afunction'.
        """


class SessionDataInterface(Interface.Base):
    """
    Session Data Object interface.

    A Session Data Object is an object stored in the Session Data
    Container related to a Session Data Manager.  Its instances are
    frequently manipulated in application code.

    All of these methods are public.
    """

    def getId(self):
        """
        Returns the id of the session data object.  In the current
        implementation, this is the same as the token value associated
        with the data object.
        """

    def set(self, k, v):
        """
        Set item with key k to value v.
        """

    def get(self, k, default):
        """
        Return the vaue associated with the key k.  If the key k
        does not exist, return None or default if passed in.
        """
        
    def __len__(self):
        """
        Returns number of items in the data object.
        """

    def __getitem__(self, k, default=None):
        """
        Returns the item associated with key k or default if item associated
        with key k doesn't exist.
        """

    def __setitem__(self, k, v):
        """
        Sets item with key k to value v.
        """

    def has_key(self, k):
        """
        Returns true if a value for key k exists.
        """
        
    def items(self):
        """
        Returns list of tuples in format (k,v) for all items in data object.
        """

    def values(self):
        """
        Returns list of values of items in data object.
        """

    def keys(self):
        """
        Returns list of keys of items in data object.
        """

    def delete(self, k):
        """
        Deletes the item in the session data object referenced by key 'k'.
        """
        
    def getId(self):
        """
        Returns the data object's id.
        """

    def getCreated(self):
        """
        Returns the time-since-epoch in float seconds respective to the
        data object's creation date.
        """

    def getLastAccessed(self):
        """
        Returns the time-since-epoch in float seconds respective to the
        data object's 'last accessed' time (the last time the object
        was retrieved from its session data container).
        """

    def invalidate(self):
        """
        Invalidates the session data object.
        """

    def clear(self):
        """
        Clears all items from the session data object.
        """

    def update(self, d):
        """
        Updates the session data object with the keys and values from
        mapping object 'd'
        """

    def copy(self):
        """
        Returns a (shallow) copy of the key/value pairs from the
        session data object in a dictionary.
        """

class SessionDataContainerInterface(Interface.Base):
    """
    Zope Session Data Container interface.  A session data container stores
    session data objects related to one or more Session Data Managers.
    
    A Session Data Container supports the Python mapping interface.
    """

    def __delitem__(self, key):
        """
        Deletes item referred to by key

        Permission required: Access session data
        """

    def __getitem__(self, key):
        """
        Returns value referred to by key.

        Permission required: Access session data
        """

    def __len__(self):
        """
        Returns number of items in container.

        Permission required: Access session data
        """

    def getLen(self):
        """
        Returns number of items in container.  Callable from TTW code.

        Permission required: Access session data
        """

    def __setitem__(self, key, v):
        """
        Sets item in container with key 'key' and value 'v'

        Permission required: Access session data
        """

    def get(self, key, default=None):
        """
        Returns value referred to by key or default if no value with key.

        Permission required: Access session data
        """

    def has_key(self, key):
        """
        Returns true if container has key.

        Permission required: Access session data
        """

    def set(self, k, v):
        """
        Set key k to value v.

        Permission required: Access session data
        """





