##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################
import sys, os, time
sys.path.insert(0, '../../..')
import makerequest
os.chdir('../../..')
import ZODB # in order to get Persistence.Persistent working
from Products.CoreSessionTracking.SessionIdManager import SessionIdManager
from Products.CoreSessionTracking.SessionDataManager import SessionDataManager
from Products.CoreSessionTracking.SessionData import SessionData
from Products.CoreSessionTracking.SessionDataManager import SessionDataManagerErr, WRITE_EVERY
from ZODB.POSException import InvalidObjectReference
from DateTime import DateTime
from unittest import TestCase, TestSuite, TextTestRunner, makeSuite

sessionidmgr = 'session_id_mgr'

def f(sdo):
    pass

class TestBase(TestCase):
    def setUp(self):
        import Zope
        self.app = makerequest.makerequest(Zope.app())
        del Zope
        timeout = self.timeout = 1
        sidmgr = SessionIdManager(sessionidmgr)
        sm=SessionDataManager(
            id='sm', path=None, timeout_mins=timeout, title='SessionThing',
            onstart='/onstartf', onend='/onendf')
        try: self.app._delObject(sessionidmgr)
        except AttributeError: pass
        try: self.app._delObject('sm')
        except AttributeError: pass
        try: self.app._delObject('byonend')
        except AttributeError: pass
        self.app._setObject(sessionidmgr, sidmgr)
        self.app._setObject('sm', sm)
        admin = self.app.acl_users.getUser('admin')
        if admin is None:
            raise "Need to define an 'admin' user before running these tests"
        admin = admin.__of__(self.app.acl_users)
        self.app.sm.changeOwnership(admin)

    def tearDown(self):
        get_transaction().abort()
        self.app._p_jar.close()
        self.app = None
        del self.app

class TestSessionManager(TestBase):
    def testHasId(self):
        assert self.app.sm.id == 'sm'

    def testHasTitle(self):
        assert self.app.sm.title == 'SessionThing'

    def testGetSessionDataNoCreate(self):
        sd = self.app.sm.getSessionData(0)
        assert sd is None, repr(sd)

    def testGetSessionDataCreate(self):
        sd = self.app.sm.getSessionData(1)
        assert sd.__class__ is SessionData, repr(sd)

    def testHasSessionData(self):
        sd = self.app.sm.getSessionData()
        assert self.app.sm.hasSessionData()

    def testNewSessionDataObjectIsValid(self):
        sdType = type(SessionData(1))
        sd = self.app.sm.getSessionData()
        assert type(getattr(sd, 'aq_base', sd)) is sdType
        assert not hasattr(sd, '_invalid')

    def testInvalidateSessionDataObject(self):
        sd = self.app.sm.getSessionData()
        sd.invalidate()
        assert hasattr(sd, '_invalid')
        
    def testSessionTokenIsSet(self):
        sd = self.app.sm.getSessionData()
        mgr = getattr(self.app, sessionidmgr)
        assert mgr.hasToken()

    def testBadExternalSDCPath(self):
        self.app.sm.REQUEST['REQUEST_METHOD'] = 'GET' # fake out webdav
        self.app.sm.setSessionDataContainerPath('/fudgeffoloo')
        try:
            self.app.sm.getSessionData()
        except SessionDataManagerErr:
            pass
        else:
            assert 1 == 2, self.app.sm.getSessionDataContainerPath()

    def testInvalidateSessionDataObject(self):
        sd = self.app.sm.getSessionData()
        sd.invalidate()
        assert not self.app.sm.hasSessionData()

    def testGhostUnghostSessionManager(self):
        get_transaction().commit()
        sd = self.app.sm.getSessionData()
        sd.set('foo', 'bar')
        self.app.sm._p_changed = None
        get_transaction().commit()
        assert self.app.sm.getSessionData().get('foo') == 'bar'

    def testSubcommit(self):
        sd = self.app.sm.getSessionData()
        sd.set('foo', 'bar')
        assert get_transaction().commit(1) == None
        
    def testForeignObject(self):
        self.assertRaises(InvalidObjectReference, self._foreignAdd)

    def _foreignAdd(self):
        ob = self.app.sm
        sd = self.app.sm.getSessionData()
        sd.set('foo', ob)
        get_transaction().commit()

class TestTimeoutRelated(TestBase):
    def testLastAccessed(self):
        sdo = self.app.sm.getSessionData()
        la1 = sdo.getLastAccessed()
        time.sleep(WRITE_EVERY + 1)
        sdo = self.app.sm.getSessionData()
        assert sdo.getLastAccessed() > la1

    def testOnStart(self):
        self.app.sm.setOnStartPath('/onstartf')
        sdo = self.app.sm.getSessionData()
        now = DateTime()
        k = sdo.get('a')
        assert type(k) == type(now)
        assert k <= now

    def testOnEnd(self):
        self.app.sm.setOnEndPath('/onendf')
        sdo = self.app.sm.getSessionData()
        timeout = self.timeout * 60
        time.sleep(timeout + (timeout * .33))
        sdo = self.app.sm.getSessionData()
        assert hasattr(self.app, 'byonend')

if __name__ == '__main__':
    suite = makeSuite(TestTimeoutRelated, 'test')
    suite2 = makeSuite(TestSessionManager, 'test')
    runner = TextTestRunner()
    runner.run(suite2)







